function PreClassification
% Builds the performance matrix, the portfolios, the training/test sets and 
% saves the suitable results before the SATzilla classification starts.

% Given an instance name, returns the corresponding ID.
function Index = instToIndex(name)
    Index = find(cellfun(@(x) strcmp(x, name), INSTs));
end

% Given a solver name, returns the corresponding ID.
function Index = solverToIndex(solver)
    Index = find(cellfun(@(x) strcmp(x, solver), SOLVERs));
end

% Build the performance matrix P so that P(i, j) is the runtime of j-th
% solver on the i-th instance.
perf_file = '/home/roberto/new_experiments/SATzilla/perf_matrix_csp.csv';
fid = fopen(perf_file);
perf = textscan(fid, '%s%s%s%s%s%s%s%s%s%s%s%s%s%s');
NUM_INST = 4642;
NUM_SOLV = 12;
INSTs = cell(NUM_INST, 1);
SOLVERs = cell(NUM_SOLV + 1, 1);
P = zeros(NUM_INST, NUM_SOLV + 1);
for i = 1:NUM_INST
    for j = 1:(NUM_SOLV + 1)
        INSTs{i} = perf{1}{i + 1};
          if i == 1
              SOLVERs{j} = perf{j + 1}{1};
          end
          P(i, j) = eval(perf{j + 1}{i + 1});
    end
end
%Build the portfolios for each different size.
port_file = '/home/roberto/new_experiments/SATzilla/portfolios_list_csp.csv';
fid = fopen(port_file); 
port = textscan(fid, '%s%s%s%s%s%s%s%s%s%s%s%s');
PORTFOLIO = cell(NUM_SOLV - 1, 1);
for p = 2:NUM_SOLV - 1
    id_list = zeros(p + 1, 1);
      for q = 1:p
          id_list(q) = solverToIndex(port{q}{p - 1});
      end
%      % 13 = solverToIndex('nosolver');
      id_list(p + 1) = 13; 
      PORTFOLIO{p} = id_list;
end

% Build the matrices corresponding to the 25 training/test sets.
X_TRAIN_NAMES = cell(5, 1);
X_TRAIN_IDs = cell(5, 1);
X_TRAIN = cell(5, 1);
X_TEST_NAMES = cell(5, 1);
X_TEST = cell(5, 1);
NUM_FEAT = 143;
MAX_TRAIN = 3714;
MAX_TEST = 929;
feat_path = '/home/roberto/new_experiments/SATzilla/features/';
for i = 1:5
   for j = 1:5        
       train_file = strcat(feat_path, 'train_rep', num2str(i), '_fold', num2str(j), '.features');
       fid = fopen(train_file);
       train = textscan(fid,  '%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s');
       names = train{1};
       X_TRAIN_NAMES{i}{j} = names;
       n = length(names);
       ids = zeros(n, 1);
       for k = 1:n
           ids(k) = instToIndex(names{k});
       end
       X_TRAIN_IDs{i}{j} = ids;
       s = size(names);
       xTrain = zeros(MAX_TRAIN, NUM_FEAT);
       for h = 1:s(1)
           for k = 1:NUM_FEAT
               xTrain(h, k) = eval(train{k + 1}{h});
           end
       end
       X_TRAIN{i}{j} = xTrain;
       
       test_file = strcat(feat_path, 'test_rep', num2str(i), '_fold', num2str(j), '.features');
       fid = fopen(test_file);
       test = textscan(fid,  '%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s');
       names = test{1};
       s = size(test{1});
       xTest = zeros(MAX_TEST, NUM_FEAT);
       for h = 1:s(1)
           for k = 1:NUM_FEAT
               xTest(h, k) = eval(test{k + 1}{h});
           end
       end
       X_TEST{i}{j} = xTest;
   end
end

% Save the results.
save('PreClassification', 'P', 'PORTFOLIO', 'X_TRAIN', 'X_TRAIN_IDs');
save('PrePredictions', 'X_TEST', 'SOLVERs'); 

end