'''
Computes statistics for SATzilla
'''

import csv
import sys
import os
import json

BACKUP_SOLVER = 'minisatid'
FEAT_SOLVER = 'gecode'
TIMEOUT = 1800.0
NUM_FEAT = 144
NUM_SOLVER = 12
FEAT_PATH = '/public/xcsp_instances/new/scripts/classifiers/csp/SATzilla/features/'
PRED_PATH = '/public/xcsp_instances/new/scripts/classifiers/csp/SATzilla/predictions/'
infotimes_file = '/public/xcsp_instances/new/solvers/csp/fixed/infotimes_csp'
bestsolvers_file = '/public/xcsp_instances/new/solvers/csp/fixed/best_solvers'

def main(args):
  with open(infotimes_file, 'r') as infile:
    infotimes = json.load(infile)
  with open(bestsolvers_file, 'r') as infile:
    best_solvers = json.load(infile)
  stats_writer = csv.writer(open(PRED_PATH + 'stats_SATzilla', 'wb'),  delimiter = '|')
  for n in range(2, NUM_SOLVER):
    counter = 0
    total_time = 0.0
    solved_instances = 0
    pred_writer = csv.writer(open(PRED_PATH + str(n) + 'solvers/Pred_' + str(n) + 'solvers', 'wb'),  delimiter = '|')
    for rep in range(1, 6):
      for fold in range(1, 6):
        suffix = 'rep' + str(rep) + '_fold' + str(fold)
        feat = FEAT_PATH + 'test_' + suffix + '.features'
        test = PRED_PATH + str(n) + 'solvers/test_' + str(n) + 'solvers_' + suffix
        feat_reader = csv.reader(open(feat, 'rb'),  delimiter = ' ')
        test_reader = csv.reader(open(test, 'rb'),  delimiter = ' ')
        for row in feat_reader:
          counter += 1
          inst = row[0]
          solv = test_reader.next()[0]
          if solv == 'nosolver':
            solv = BACKUP_SOLVER
          it = infotimes[inst][solv]
          if solv == FEAT_SOLVER:
            time = float(best_solvers[inst]['feat']) + it[1] - float(best_solvers[inst]['mzn2fzn'])
          else:
            time = float(best_solvers[inst]['feat']) + it[1]
          if it[0] != "sat" and it [0] != "unsat":
            if it[0] != "timeout":
              itb = infotimes[inst][BACKUP_SOLVER]
              time += itb[1]
              if time < TIMEOUT and (itb[0] == "sat" or itb[0] == "unsat"):
                solved_instances += 1
                succ = 1
              else:
                time = TIMEOUT
                succ = 0
            else:
              time = TIMEOUT
              succ = 0
          else:
            if time < TIMEOUT:
              solved_instances += 1
              succ = 1
            else:
              time = TIMEOUT
              succ = 0
          total_time += time
          pred_writer.writerow([inst, rep, fold, succ, time, solv])
    avg = str(total_time / counter)
    psi = str(100 * float(solved_instances) / counter)
    stats_writer.writerow([n, avg, psi])

if __name__ == '__main__':
    main(sys.argv[1:])
